-------------------------------------------------------------------------------
--
-- Copyright (C) 2011
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: I2C commander settings for the UniBoard2 I2C SENS bus
--          sensors.
-- Remark:
-- . Adapted from i2c_commander_unbh_pkg
--   reduce the number of protocol lists to 4

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE work.i2c_pkg.ALL;
USE work.i2c_commander_pkg.ALL;
USE work.i2c_dev_unb2_pkg.ALL;    -- slave addresses and commands for all unb2 i2c interfaces


PACKAGE i2c_commander_unb2_sens_pkg IS

  ------------------------------------------------------------------------------
  -- Local constants (with prefix 'k_')
  ------------------------------------------------------------------------------
  
  -- Define the protocol lists for the commander (c_i2c_cmdr_max_nof_protocols=16)
  CONSTANT k_protocol_list_0      : t_nat_natural_arr := c_i2c_unb2_sens_max1617_protocol_list_read_config;
  CONSTANT k_protocol_list_1      : t_nat_natural_arr := c_i2c_unb2_sens_max1617_protocol_list_read_temp;
  CONSTANT k_protocol_list_2      : t_nat_natural_arr := c_i2c_unb2_sens_protocol_list_read_all;  
  CONSTANT k_protocol_list_3      : t_nat_natural_arr := c_i2c_cmdr_protocol_list_end;
  
  CONSTANT k_protocol_ram_init : t_nat_natural_arr := k_protocol_list_0  &
                                                      k_protocol_list_1  &
                                                      k_protocol_list_2  &
                                                      k_protocol_list_3;
                                                               
  -- Define the corresponding mask words for the result data
  CONSTANT k_expected_mask_0      : STD_LOGIC_VECTOR := c_i2c_unb2_sens_max1617_expected_mask_read_config;
  CONSTANT k_expected_mask_1      : STD_LOGIC_VECTOR := c_i2c_unb2_sens_max1617_expected_mask_read_temp;
  CONSTANT k_expected_mask_2      : STD_LOGIC_VECTOR := c_i2c_unb2_sens_expected_mask_read_all;
  CONSTANT k_expected_mask_3      : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_4      : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_5      : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_6      : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_7      : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_8      : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_9      : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_10     : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_11     : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_12     : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_13     : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_14     : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;
  CONSTANT k_expected_mask_15     : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_end;

  CONSTANT k_expected_mask_arr : t_slv_32_arr(0 TO c_i2c_cmdr_max_nof_protocols-1) := (k_expected_mask_0,
                                                                                       k_expected_mask_1,
                                                                                       k_expected_mask_2,
                                                                                       k_expected_mask_3,
                                                                                       k_expected_mask_4,
                                                                                       k_expected_mask_5,
                                                                                       k_expected_mask_6,
                                                                                       k_expected_mask_7,
                                                                                       k_expected_mask_8,
                                                                                       k_expected_mask_9,
                                                                                       k_expected_mask_10,
                                                                                       k_expected_mask_11,
                                                                                       k_expected_mask_12,
                                                                                       k_expected_mask_13,
                                                                                       k_expected_mask_14,
                                                                                       k_expected_mask_15);

                                                                                       
  -- Define the corresponding expected nof read data
  CONSTANT k_nof_result_data_0    : NATURAL := c_i2c_unb2_sens_max1617_nof_result_data_read_config;
  CONSTANT k_nof_result_data_1    : NATURAL := c_i2c_unb2_sens_max1617_nof_result_data_read_temp;
  CONSTANT k_nof_result_data_2    : NATURAL := c_i2c_unb2_sens_nof_result_data_read_all;
  CONSTANT k_nof_result_data_3    : NATURAL := c_i2c_cmdr_nof_result_data_none;
  
  CONSTANT k_nof_result_data_arr : t_nat_natural_arr := (k_nof_result_data_0,
                                                         k_nof_result_data_1,
                                                         k_nof_result_data_2,
                                                         k_nof_result_data_3);
                                                         
  -- Define the corresponding protocol list offsets
  CONSTANT k_protocol_ofs_0  : NATURAL := 0;
  CONSTANT k_protocol_ofs_1  : NATURAL := k_protocol_list_0'LENGTH  + k_protocol_ofs_0;
  CONSTANT k_protocol_ofs_2  : NATURAL := k_protocol_list_1'LENGTH  + k_protocol_ofs_1;
  CONSTANT k_protocol_ofs_3  : NATURAL := k_protocol_list_2'LENGTH  + k_protocol_ofs_2;
  CONSTANT k_protocol_ofs_4  : NATURAL := 0 + k_protocol_ofs_3;
  CONSTANT k_protocol_ofs_5  : NATURAL := 0 + k_protocol_ofs_4;
  CONSTANT k_protocol_ofs_6  : NATURAL := 0 + k_protocol_ofs_5;
  CONSTANT k_protocol_ofs_7  : NATURAL := 0 + k_protocol_ofs_6;
  CONSTANT k_protocol_ofs_8  : NATURAL := 0 + k_protocol_ofs_7;
  CONSTANT k_protocol_ofs_9  : NATURAL := 0 + k_protocol_ofs_8;
  CONSTANT k_protocol_ofs_10 : NATURAL := 0 + k_protocol_ofs_9;
  CONSTANT k_protocol_ofs_11 : NATURAL := 0 + k_protocol_ofs_10;
  CONSTANT k_protocol_ofs_12 : NATURAL := 0 + k_protocol_ofs_11;
  CONSTANT k_protocol_ofs_13 : NATURAL := 0 + k_protocol_ofs_12;
  CONSTANT k_protocol_ofs_14 : NATURAL := 0 + k_protocol_ofs_13;
  CONSTANT k_protocol_ofs_15 : NATURAL := 0 + k_protocol_ofs_14;

  CONSTANT k_protocol_ofs_arr : t_natural_arr(0 TO c_i2c_cmdr_max_nof_protocols-1) := (k_protocol_ofs_0,
                                                                                       k_protocol_ofs_1,
                                                                                       k_protocol_ofs_2,
                                                                                       k_protocol_ofs_3,
                                                                                       k_protocol_ofs_4,
                                                                                       k_protocol_ofs_5,
                                                                                       k_protocol_ofs_6,
                                                                                       k_protocol_ofs_7,
                                                                                       k_protocol_ofs_8,
                                                                                       k_protocol_ofs_9,
                                                                                       k_protocol_ofs_10,
                                                                                       k_protocol_ofs_11,
                                                                                       k_protocol_ofs_12,
                                                                                       k_protocol_ofs_13,
                                                                                       k_protocol_ofs_14,
                                                                                       k_protocol_ofs_15);


  -- RAM sizes
  CONSTANT k_protocol_ram_nof_dat : NATURAL := ceil_div(k_protocol_ram_init'LENGTH, c_i2c_cmdr_mem_block_sz) * c_i2c_cmdr_mem_block_sz;
  CONSTANT k_protocol_ram_adr_w   : NATURAL := ceil_log2(k_protocol_ram_nof_dat);  
  CONSTANT k_result_adr_w         : NATURAL := k_protocol_ram_adr_w-1;  -- assume < 1 result byte per 2 protocol bytes
  CONSTANT k_result_nof_dat       : NATURAL := 2**k_result_adr_w;
  
  -- Commander settings
  CONSTANT k_nof_protocols        : NATURAL := 3;  -- Must be >= actually used nof protocol lists in k_protocol_ram_init and <= c_i2c_cmdr_max_nof_protocols=16 protocols.
  CONSTANT k_nof_result_data_max  : NATURAL := 32;  -- Must be >= the maximum nof read bytes in any protocol list in k_protocol_ram_init
  CONSTANT k_result_cnt_w         : NATURAL := k_result_adr_w;
  
  
  ------------------------------------------------------------------------------
  -- Global constants (with prefix 'c_i2c_cmdr_unbh_')
  ------------------------------------------------------------------------------
  
  CONSTANT c_i2c_cmdr_unb2_sens_protocol_ram_init   : t_nat_natural_arr := k_protocol_ram_init;
  CONSTANT c_i2c_cmdr_unb2_sens_nof_result_data_arr : t_nat_natural_arr := k_nof_result_data_arr;
  
  CONSTANT c_i2c_cmdr_unb2_sens_i2c_mm              : t_c_i2c_mm := (c_i2c_control_adr_w,
                                                                k_protocol_ram_adr_w,
                                                                k_protocol_ram_nof_dat,
                                                                k_result_adr_w,
                                                                k_result_nof_dat);
  
  CONSTANT c_i2c_cmdr_unb2_sens_protocol_commander  : t_c_i2c_cmdr_commander := (k_nof_protocols,
                                                                            k_protocol_ofs_arr,
                                                                            k_expected_mask_arr,
                                                                            k_result_cnt_w,
                                                                            k_nof_result_data_max);

END i2c_commander_unb2_sens_pkg;


PACKAGE BODY i2c_commander_unb2_sens_pkg IS
END i2c_commander_unb2_sens_pkg;
