-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE common_lib.common_pkg.ALL;
USE work.i2c_smbus_pkg.ALL;
USE work.i2c_dev_max1617_pkg.ALL;
USE work.i2c_dev_max6652_pkg.ALL;
USE work.i2c_commander_pkg.ALL;


PACKAGE i2c_dev_unb_pkg IS

  -- I2C slave addresses of the devices on the I2C bus on UniBoard
  CONSTANT I2C_UNB_MAX1617_ADR     : NATURAL := MAX1617_ADR_LOW_LOW;   -- temperature sensor, slave address is "0011000"
  CONSTANT I2C_UNB_MAX6652_ADR     : NATURAL := MAX6652_ADR_GND;   -- temperature-voltage sensor, slave address is "0010100"
  
  CONSTANT c_i2c_unb_temp_high     : NATURAL := 85;
  
  -- Commander protocol lists for UNB control
  
  -- > Temperature sensor:
  --       . c_i2c_unb_protocol_list_read_config
  --       . c_i2c_unb_protocol_list_read
  
  CONSTANT c_i2c_unb_max1617_expected_mask_read_config   : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_read_one_byte;
  CONSTANT c_i2c_unb_max1617_nof_result_data_read_config : NATURAL := c_i2c_cmdr_nof_result_data_read_one_byte;
  CONSTANT c_i2c_unb_max1617_protocol_list_read_config   : t_nat_natural_arr := (
    SMBUS_READ_BYTE,  I2C_UNB_MAX1617_ADR, MAX1617_CMD_READ_REMOTE_TEMP,
    SMBUS_WRITE_BYTE, I2C_UNB_MAX1617_ADR, MAX1617_CMD_WRITE_CONFIG, MAX1617_CONFIG_ID+MAX1617_CONFIG_THERM,
    SMBUS_WRITE_BYTE, I2C_UNB_MAX1617_ADR, MAX1617_CMD_WRITE_REMOTE_HIGH, c_i2c_unb_temp_high,
    SMBUS_C_END
  );
  
  CONSTANT c_i2c_unb_max1617_expected_mask_read_temp   : STD_LOGIC_VECTOR := c_i2c_cmdr_expected_mask_read_one_byte;
  CONSTANT c_i2c_unb_max1617_nof_result_data_read_temp : NATURAL := c_i2c_cmdr_nof_result_data_read_one_byte;
  CONSTANT c_i2c_unb_max1617_protocol_list_read_temp   : t_nat_natural_arr := (
    SMBUS_READ_BYTE, I2C_UNB_MAX1617_ADR, MAX1617_CMD_READ_REMOTE_TEMP,
    SMBUS_C_END
  );
  
  -- > Temperature-Voltage sensor:
  --       . c_i2c_unb_max6652_protocol_list_read_config
  
  CONSTANT c_i2c_unb_max6652_expected_mask_read_config   : STD_LOGIC_VECTOR := RESIZE_UVEC("001010101", c_word_w);
  CONSTANT c_i2c_unb_max6652_nof_result_data_read_config : NATURAL := 4;
  CONSTANT c_i2c_unb_max6652_protocol_list_read_config   : t_nat_natural_arr := (
    SMBUS_READ_BYTE , I2C_UNB_MAX6652_ADR, MAX6652_REG_READ_VIN_2_5,
    SMBUS_READ_BYTE , I2C_UNB_MAX6652_ADR, MAX6652_REG_READ_VIN_3_3, 
    SMBUS_READ_BYTE , I2C_UNB_MAX6652_ADR, MAX6652_REG_READ_VCC, 
    SMBUS_READ_BYTE , I2C_UNB_MAX6652_ADR, MAX6652_REG_READ_TEMP, 
    SMBUS_WRITE_BYTE, I2C_UNB_MAX6652_ADR, MAX6652_REG_CONFIG, MAX6652_CONFIG_LINE_FREQ_SEL+MAX6652_CONFIG_START,
    SMBUS_C_END
  );
  
END PACKAGE;
